$nomod51									;Unterdrckt die Standardbefehle des 8051

$include (C8051F320.inc)	;C8051F320-spezifische Daten einbeziehen
											
$title (SPI_test2)

;-----------------------------------------------
;Programmbeschreibung
;-----------------------------------------------

; Mit diesem Programm wird ein einmaliger Kommunikationstest durchgefhrt,
; angenhert dem Testprogramm von ACAM (fr 8051-uC in C).
; und ein Kommunikationstest des SPI durchgefhrt

; Das Programm vollzieht sich vollautomatisch, ohne Zutun.
; Es mu nur gestartet werden.

; Ergebnis dieses Test:
; Der Inhalt der hchsten 8 Bit des Registers 1 sollte
; vom TDC nach Leseaufforderung zurck gegeben werden.
; Siehe Doku TDC-GP2 Tabelle 2.6.2 "Read Registers"


; Zustzlich wird zwar auf einen Interrupt seitens des TDC eingegangen,
; doch diesem Aufruf hat noch keine wirklichen Folgen. 
; Er ist fr eine sptere Programmerweiterung im Sinne der
; eigentlichen Aufgabe des Boards gedacht.

;-------------
;Projekt:					Flankengenerator II
;Vorlesung/Labor:	Hybride Integration - Labor
;									Prof. Dipl. Wirtsch.-Ing. Fritz J. Neff
;Semester:				MTB7 - Hochschule Karlsruhe 

;Erstellt am:12.06.2010
;Programmiert:
;Boris Sembritzki, sebo0011@hs-karlsruhe.de

;Betreut durch: Dipl.-Ing. Bernhard Beck

;-------------
;Verwendete Register und Variablen:
;R1	Empfangener Inhalt vom SPI
;R2	Inhalt zum Senden per SPI
;R4 Verzgerungsschleife (50ns)
;R5 Zhler fr TDC-Hardwarereset
;R6 Aktuell eingeleses Datenbyte (SPI) 
;R7 Dummy-Variable

;-------------
;Zur Kommentierung:
;GROSS geschriebene Labels betreffen den Mikrocontroller
;klein geschriebene Labels sind TDC-eigene Befehl / Konfigurierung

;-------------
;nderungen:

; Gendert am :...
; Kommentar: ...

;-----------------------------------------------
;Hardwarebeschreibung
;-----------------------------------------------
; Anwendungszweck des Boards:
;		Erproben des TDC_GP2 (Time to Digital Converter) der Firma ACAM

; Verwendeter Mikrocontroller:
;		-C8051F302 von SiliconLabs

; Mensch-Board-Kommunikation:
; 	Das Board hat keine Taster oder LED
;		Die Verbindung zum Nutzer wird einzig ber UART/Hyperterminal realisiert

; Weitere fr die Programmierung relevante ICs:
;		-TDC_GP2 (Time to Digital Converter
;		-MAX3222E (fr EIA232)

; Kommunikation zwischen TDC-GP2 und Mikrocontroller:
;		-SPI (Mikrocontroller = Singlemaster)
;		-ber eine Vielzahl von Pins (siehe Kommentar zur Pinbelegung)

; Kommunikation zwischen MAX3222E und Mikrocontroller
;		-Neben der Konfiguration ist nur der Enablepin (P1.1)
;    fr die Programmierung auffllig

; Schnittstellen des Boards, die im Programmtext aufgegriffen werden knnen:
;		-UART
;		-Flankeneingnge (fhren zum TDC): Start und Stop1
;		 Anmerkung: der Pin Stop2 des TDC ist ungenutzt 

; Pinbelegung am Mikrocontroller:
;		-P0.1 Enable MAX3222E           - Low aktiv
;		-P0.4 ursprnglich UART, hier nicht verwendet
;		-P0.5 ursprnglich UART, hier nicht verwendet
;		-P0.6 RSTIN (TDC-ResetInput)		- Low aktiv
;		-P0.7 INTN (Interrupt vom TDC)	- Low aktiv
;		-P1.0 SPI (zum TDC - MOSI)
;		-P1.1 SPI (zum TDC - SCK)
;		-P1.2 SPI (zum TDC - MISO)
;		-P1.3 SPI (zum TDC - SSN)				- Low aktiv
;		-P1.4 EN_Start (TDC)						- High aktiv
;		-P1.5 EN_Stop1 (TDC)						- High aktiv
;		-P1.6 EN_Stop2 (TDC) 						- High aktiv
;		-P1.7 Sysclock (Clock-Ausgang fr TDC)

; Sysclock nach auen gefhrt (P1.7) fr den TDC-GP2:
;		Soll (laut Datenblatt S.26): 2...8MHz
;		Ist: 6MHz
;			(Optional wren auch 3MHz mglich)

;-----------------------------------------------
;Initialisierungsteil fr allgemeine Konstanten und Variablen
;-----------------------------------------------
CSEG AT 0H
jmp INIT
Verzug_50ns EQU 05h	;fr NSS_Reset_50ns
;DUMMY SET 0								;Dummy-Variable 


;-----------------------------------------------
;Interrrupt Vektoren
;-----------------------------------------------

;--------------
;External Interrupt (s. S.88) - Pin0.7
; 560ns oder (wenn kalibriert) 4,6us nach der letzten aufsteigenden Flanke
; (am Stop-Pin des TDC) sendet der TDC-GP2 ein Interruptsignal an Pin 0.7  
; (des uC).
ORG 0003h
call ISR_INT0
reti

;--------------
;SPI-Interrupt
ORG 0033h
call ISR_SPI
reti

;-----------------------------------------------
;Initialisierungsteil fr On-Chip-Peripherie
;-----------------------------------------------
ORG 100H
INIT:
;Watchdog
anl PCA0MD,#0BFH	; Watchdog hier zustzlich ausschalten, sonst strend
call Init_Device	; Aufruf zur Initialisierung der Controllerfunktionen

;Pins
setb P0.1					; Enable MAX3222E - nun nicht aktiv
setb P0.6					; TDC-Reset (RSTIN)
setb P1.4					; EN_Start (TDC) - rckgesetzt
setb P1.5					; EN_Stop1 (TDC) - rckgesetzt
clr P1.6					; EN_Stop2 (TDC) - Es gibt fr Stop2 keinen Anschlu nach auen 

;Schnittstellen
call init_transaktion ;50ns Reset ber NSS-Pin; WCOL-Interruptflag rcksetzten
clr SPIF					; SPI0-Interruptflag rcksetzten
clr NSSMD0				; NSS-Pin rckgesetzt (d.h. kein Reset, keine Trennung zum Master)

;Register
mov R1,#30h				; Empfangsregister fr SPI auf Adresse D:#30h setzten
mov R2,#00h				; Senderegister fr SPI leeren
mov R5,#30h				; willkrlich gesetzte Lnge fr TDC-Hardwarereset
mov R6,#03h				; Anzahl zu lesender Byte
mov R7,#00h				; Dummy-Variable/Register

;-----------------------------------------------
;Programmschleife
;-----------------------------------------------
ABFRAGE:

;------------TDC-Interrupt
mov C,P0.7				; Interrupt bzgl. Taste funktioniert immer nur auf P3.2 oder P3.3
mov P3.2,C				; Daher mu der Zustand des P0.7 auf P3.2 weitergeleitet werden.
	
;------------Befehlfolge an TDC ber SPI
;						 (siehe Bsp. im Datenblatt S.34,Kap.6.1.3)

	call tdc_hardware_reset
	call R1_ADRESSEN_LEEREN
	call init_transaktion
	call tdc_schreibe_array
	call init_transaktion		;50ms NSS-Reset zwischen dem Schreiben und Lesen (s.TDC-Doku)
	call tdc_lesen_reg5			;Kommunikatationstest
	setb NSSMD0							;Kommunikation mit TDC beendet

jmp $ ;Programm soll nicht in Endlosschleife laufen (Keine Rckfhrung auf ABFRAGE)
;jmp ABFRAGE

;------------------------------------------------
;CALL-Aufrufe zur Programmschleife
;------------------------------------------------

tdc_hardware_reset:		; Adressen leeren, wo spter SPI-Lesedaten gelagert werden sollen.
	mov R5,#30h
	clr P0.6						; TDC-Hardwarereset ausgelst

	RESET_SCHLEIFE:
	djnz R5,RESET_SCHLEIFE
	setb P0.6						; TDC-Hardwarereset beendet
ret

;---------------
tdc_schreibe_array:
;	clr ESPI0						; Interrupts ber IE.6 ausgeschalten (siehe SilLab SPI-Master-Bsp)
	mov R2,#81h					; TDC-Befehl: Schreibe in TDC-Register1
	call SPI_SENDEN			; Schreibbefehl abschicken

											; Die TDC-Register sind nicht bitadressierbar.
											; Es mu immer ein vollstndiges Register beschrieben werden.
	mov R2,#0D4h			
	call SPI_SENDEN
	mov R2,#09h
	call SPI_SENDEN							
	mov R2,#22h
	call SPI_SENDEN
	clr SPIF						
	;jnb SPIF,$					; Erst wenn letztes Datenbyte den Slave (TDC) erreicht weiter
											; (siehe SilLab SPI-Master-Bsp)
											; Vermutung: Flag funktioniert auch, wenn Interrupt ausgeschaltet
	clr SPIF						; 
;	clr WCOL
;	setb ESPI0					; Interrupts ber IE.6 wieder einschalten
ret

;--------------
SPI_SENDEN:						; Der SPI0DAT-Puffer ist 8Bit gro. Er wird zum Senden und Empfangen bentzt.
	mov A,R2						; Inhalt von R2 senden
	mov SPI0DAT,A
	jnb TXBMT,$					; Erst wenn SPI-Datenpuffer voll weiter (SPI-Interrupt)
	mov A,SPI0DAT				; SPI-Datenpuffer leeren
	mov R7,A
	clr WCOL
ret

;--------------
R1_ADRESSEN_LEEREN:		; Adressen leeren, wo spter SPI-Lesedaten gelagert werden sollen.
	mov R6,#03h
 	mov R1,#30h	

	LEERSCHLEIFE:
	mov @R1,#00h				; Inhalt der Adresse auf die R1 ausgerichtet ist lschen
	inc R1							; R1 auf nchste Adresse setzten
	djnz R6,LEERSCHLEIFE
ret

;--------------
tdc_lesen_reg5:
	SPI_LESEN_INIT:
	mov R1,#30h					; Initialisierung erste Ablageadresse fr SPI-Empfangsdaten
	mov R6,#03h					;	Initialisierung Anzahl zu empfangender Datenbyte

	tdc_lesebefehl:
	mov R2,#0B5h				; Lesebefehl fr TDC-Register 5
	mov A,R2						; Lesebefehl abschicken
	mov SPI0DAT,A
	jnb SPIF,$
	mov A,SPI0DAT				; SPI0DAT lesen (Akku als Dummy),da kein WCOL-Interrupt, 
	mov R7,A

	LESESCHLEIFE:
		mov SPI0DAT,#0B5h	; Lesebefehl fr TDC-Register 5
		jnb SPIF,$				; Erst wenn SPI-Datenpuffer voll weiter (SPI-Interrupt)
		mov A,SPI0DAT			;	Inhalt SPI-Puffer in Lese-Register R2
		mov R1,A
		inc R1						; Ablageadresse fr SPI-Empfangsdaten um eins erhhen
	djnz R6,LESESCHLEIFE		
ret

;--------------
init_transaktion:
	mov A,SPI0DAT				; SPI-Datenpuffer leeren
	mov R7,A
	clr WCOL						; Write Collision Flag rcksetzen
	mov R4,#Verzug_50ns	; Schleife neu auffllen
	setb NSSMD0					; High-aktiver Reset fr SPI
jmp verzoegerungsschleife

verzoegerungsschleife:
	djnz R4,verzoegerungsschleife		;Verzgerungsschleife fr Haltedauer des NSS-Reset
	clr NSSMD0					; NSS_Reset z.B. fr zwischen jedem Lesen und Schreiben
											; Nachfolgende Befehle nun nicht von vorangegangenen Zustnden abhngig.
ret

;-----------------------------------------------
;Interrupt Service Routinen
;-----------------------------------------------

ISR_INT0:		;Interrupt ausgelst durch TDC
	push ACC	;Akku sichern
	push PSW	;Programm Status Word sichern

		nop			;Diese ISR wird noch nicht wirklich verwendet.

	pop PSW
	pop ACC
	ret

;----------------
ISR_SPI:		;SPI fr Kommuniktation mit TDC
	push ACC	;Akku sichern
	push PSW	;Programm Status Word sichern

	jnb SPIF,ISR_SPI_SPIF
	jnb WCOL,ISR_SPI_WCOL
	jmp ISR_SPI_ENDE
		
		ISR_SPI_SPIF:

			clr SPIF;Flag mu von Hand zurck gesetzt werden
							;(Dieses Flag ist immer = logisch 1, wenn
							;ein Transfer - rein oder raus - beendet wurde.)
		
		jnb WCOL,ISR_SPI_WCOL	
		jmp ISR_SPI_ENDE

		ISR_SPI_WCOL:
			clr WCOL;Flag mu von Hand zurck gesetzt werden		
							;(Dieses Flag ist immer = logisch 1, wenn
							;der Transmit-Puffer nicht geleert wurde aber geschrieben werden soll.
							;-> In diesem Fall wird das was geschrieben werden soll vom System ignoriert)


		ISR_SPI_ENDE:
	pop PSW
	pop ACC
	ret

;-----------------------------------------------
; Konfigurierung ber Wizard
; Peripheral specific initialization functions,
; Called from the Init_Device label
;-----------------------------------------------

SPI_Init:
    mov  SPI0CFG,   #060h
    mov  SPI0CN,    #009h
    ret

Port_IO_Init:
    ; P0.0  -  Skipped,     Open-Drain, Digital
    ; P0.1  -  Skipped,     Open-Drain, Digital
    ; P0.2  -  Skipped,     Open-Drain, Digital
    ; P0.3  -  Skipped,     Open-Drain, Digital
    ; P0.4  -  Skipped,     Push-Pull,  Digital
    ; P0.5  -  Skipped,     Push-Pull,  Digital
    ; P0.6  -  Skipped,     Open-Drain, Digital
    ; P0.7  -  Skipped,     Open-Drain, Digital

    ; P1.0  -  SCK  (SPI0), Open-Drain, Digital
    ; P1.1  -  MISO (SPI0), Open-Drain, Digital
    ; P1.2  -  MOSI (SPI0), Open-Drain, Digital
    ; P1.3  -  NSS  (SPI0), Open-Drain, Digital
    ; P1.4  -  Skipped,     Open-Drain, Digital
    ; P1.5  -  Skipped,     Open-Drain, Digital
    ; P1.6  -  Skipped,     Open-Drain, Digital
    ; P1.7  -  SYSCLK,      Open-Drain, Digital
    ; P2.0  -  Unassigned,  Open-Drain, Digital
    ; P2.1  -  Unassigned,  Open-Drain, Digital
    ; P2.2  -  Unassigned,  Open-Drain, Digital
    ; P2.3  -  Unassigned,  Open-Drain, Digital

    mov  P0MDOUT,   #030h
    mov  P0SKIP,    #0FFh
    mov  P1SKIP,    #070h
    mov  XBR0,      #00Ah
    mov  XBR1,      #040h
    ret

Oscillator_Init:
    mov  OSCICN,    #081h
    ret

Interrupts_Init:
    mov  IE,        #0C8h
    ret

; Initialization function for device,
; Call Init_Device from your main program
Init_Device:
;   lcall Timer_Init
    lcall SPI_Init
    lcall Port_IO_Init
    lcall Oscillator_Init
    lcall Interrupts_Init
    ret

end
